@@ -26,8 +26,6 @@
 #include "virbuffer.h"
 #include "virfile.h"
 #include "cpu_conf.h"
-#include "domain_conf.h"
-#include "virstring.h"
 #include "virlog.h"
 
 #define VIR_FROM_THIS VIR_FROM_CPU
@@ -84,6 +82,12 @@ VIR_ENUM_IMPL(virCPUCacheMode,
               "disable",
 );
 
+VIR_ENUM_IMPL(virCPUMaxPhysAddrMode,
+              VIR_CPU_MAX_PHYS_ADDR_MODE_LAST,
+              "emulate",
+              "passthrough",
+);
+
 
 virCPUDef *virCPUDefNew(void)
 {
@@ -129,6 +133,7 @@ virCPUDefFree(virCPUDef *def)
     if (g_atomic_int_dec_and_test(&def->refs)) {
         virCPUDefFreeModel(def);
         g_free(def->cache);
+        g_free(def->addr);
         g_free(def->tsc);
         g_free(def);
     }
@@ -245,12 +250,20 @@ virCPUDefCopyWithoutModel(const virCPUDef *cpu)
     copy->threads = cpu->threads;
     copy->arch = cpu->arch;
     copy->migratable = cpu->migratable;
+    copy->sigFamily = cpu->sigFamily;
+    copy->sigModel = cpu->sigModel;
+    copy->sigStepping = cpu->sigStepping;
 
     if (cpu->cache) {
         copy->cache = g_new0(virCPUCacheDef, 1);
         *copy->cache = *cpu->cache;
     }
 
+    if (cpu->addr) {
+        copy->addr = g_new0(virCPUMaxPhysAddrDef, 1);
+        *copy->addr = *cpu->addr;
+    }
+
     if (cpu->tsc) {
         copy->tsc = g_new0(virHostCPUTscInfo, 1);
         *copy->tsc = *cpu->tsc;
@@ -321,6 +334,7 @@ virCPUDefParseXML(xmlXPathContextPtr ctxt,
     g_autoptr(virCPUDef) def = NULL;
     g_autofree xmlNodePtr *nodes = NULL;
     xmlNodePtr topology = NULL;
+    xmlNodePtr maxphysaddrNode = NULL;
     VIR_XPATH_NODE_AUTORESTORE(ctxt)
     int n;
     int rv;
@@ -421,6 +435,7 @@ virCPUDefParseXML(xmlXPathContextPtr ctxt,
     if (def->type == VIR_CPU_TYPE_HOST) {
         g_autofree char *arch = virXPathString("string(./arch[1])", ctxt);
         xmlNodePtr counter_node = NULL;
+        xmlNodePtr signature_node = NULL;
 
         if (!arch) {
             virReportError(VIR_ERR_XML_ERROR, "%s",
@@ -441,6 +456,26 @@ virCPUDefParseXML(xmlXPathContextPtr ctxt,
             return -1;
         }
 
+        if ((signature_node = virXPathNode("./signature[1]", ctxt))) {
+            if (virXMLPropUInt(signature_node, "family", 10,
+                               VIR_XML_PROP_REQUIRED | VIR_XML_PROP_NONZERO,
+                               &def->sigFamily) < 0) {
+                return -1;
+            }
+
+            if (virXMLPropUInt(signature_node, "model", 10,
+                               VIR_XML_PROP_REQUIRED,
+                               &def->sigModel) < 0) {
+                return -1;
+            }
+
+            if (virXMLPropUInt(signature_node, "stepping", 10,
+                               VIR_XML_PROP_REQUIRED,
+                               &def->sigStepping) < 0) {
+                return -1;
+            }
+        }
+
         if ((counter_node = virXPathNode("./counter[@name='tsc']", ctxt))) {
             tsc = g_new0(virHostCPUTscInfo, 1);
 
@@ -622,6 +657,21 @@ virCPUDefParseXML(xmlXPathContextPtr ctxt,
         def->cache->mode = mode;
     }
 
+    if ((maxphysaddrNode = virXPathNode("./maxphysaddr[1]", ctxt))) {
+        def->addr = g_new0(virCPUMaxPhysAddrDef, 1);
+
+        if (virXMLPropEnum(maxphysaddrNode, "mode",
+                           virCPUMaxPhysAddrModeTypeFromString,
+                           VIR_XML_PROP_REQUIRED,
+                           &def->addr->mode) < 0)
+            return -1;
+
+        if (virXMLPropInt(maxphysaddrNode, "bits", 10,
+                          VIR_XML_PROP_NONNEGATIVE,
+                          &def->addr->bits, -1) < 0)
+            return -1;
+    }
+
     *cpu = g_steal_pointer(&def);
     return 0;
 }
@@ -753,6 +803,14 @@ virCPUDefFormatBuf(virBuffer *buf,
         virBufferAsprintf(buf, "<microcode version='%u'/>\n",
                           def->microcodeVersion);
 
+    if (def->type == VIR_CPU_TYPE_HOST && def->sigFamily) {
+        virBufferAddLit(buf, "<signature");
+        virBufferAsprintf(buf, " family='%u'", def->sigFamily);
+        virBufferAsprintf(buf, " model='%u'", def->sigModel);
+        virBufferAsprintf(buf, " stepping='%u'", def->sigStepping);
+        virBufferAddLit(buf, "/>\n");
+    }
+
     if (def->type == VIR_CPU_TYPE_HOST && def->tsc) {
         virBufferAddLit(buf, "<counter name='tsc'");
         virBufferAsprintf(buf, " frequency='%llu'", def->tsc->frequency);
@@ -781,6 +839,15 @@ virCPUDefFormatBuf(virBuffer *buf,
         virBufferAddLit(buf, "/>\n");
     }
 
+    if (def->addr) {
+        virBufferAddLit(buf, "<maxphysaddr ");
+        virBufferAsprintf(buf, "mode='%s'",
+                          virCPUMaxPhysAddrModeTypeToString(def->addr->mode));
+        if (def->addr->bits != -1)
+            virBufferAsprintf(buf, " bits='%d'", def->addr->bits);
+        virBufferAddLit(buf, "/>\n");
+    }
+
     for (i = 0; i < def->nfeatures; i++) {
         virCPUFeatureDef *feature = def->features + i;
 
@@ -1043,6 +1110,15 @@ virCPUDefIsEqual(virCPUDef *src,
         return false;
     }
 
+    if ((src->addr && !dst->addr) ||
+        (!src->addr && dst->addr) ||
+        (src->addr && dst->addr &&
+         (src->addr->mode != dst->addr->mode ||
+          src->addr->bits != dst->addr->bits))) {
+        MISMATCH("%s", _("Target CPU maxphysaddr does not match source"));
+        return false;
+    }
+
     if (src->nfeatures != dst->nfeatures) {
         MISMATCH(_("Target CPU feature count %zu does not match source %zu"),
                  dst->nfeatures, src->nfeatures);
