@@ -33,15 +33,10 @@
 #include "virutil.h"
 #include "virfile.h"
 #include "virpidfile.h"
-#include "virprocess.h"
 #include "virerror.h"
 #include "virlog.h"
 #include "viralloc.h"
-#include "virconf.h"
 #include "rpc/virnetdaemon.h"
-#include "virrandom.h"
-#include "virhash.h"
-#include "viruuid.h"
 #include "virstring.h"
 #include "virgettext.h"
 #include "virdaemon.h"
@@ -859,9 +854,8 @@ int main(int argc, char **argv) {
     }
 
     if (timeout > 0) {
-        VIR_DEBUG("Registering shutdown timeout %d", timeout);
-        virNetDaemonAutoShutdown(logDaemon->dmn,
-                                 timeout);
+        if (virNetDaemonAutoShutdown(logDaemon->dmn, timeout) < 0)
+            return -1;
     }
 
     if ((virLogDaemonSetupSignals(logDaemon->dmn)) < 0) {
