@@ -45,11 +45,9 @@
 
 #include <net/if.h>
 
-#include "viralloc.h"
 #include "virlog.h"
 #include "datatypes.h"
 #include "virerror.h"
-#include "conf/domain_conf.h"
 #include "nwfilter_gentech_driver.h"
 #include "nwfilter_dhcpsnoop.h"
 #include "nwfilter_ipaddrmap.h"
