@@ -22,7 +22,6 @@
 #include <gnutls/x509.h>
 
 #include "locking/domain_lock.h"
-#include "viralloc.h"
 #include "virerror.h"
 #include "virlog.h"
 #include "virnetdevopenvswitch.h"
@@ -475,12 +474,10 @@ qemuMigrationCookieAddNetwork(qemuMigrationCookie *mig,
 
 static int
 qemuMigrationCookieAddNBD(qemuMigrationCookie *mig,
-                          virQEMUDriver *driver,
                           virDomainObj *vm)
 {
     qemuDomainObjPrivate *priv = vm->privateData;
     g_autoptr(GHashTable) stats = virHashNew(g_free);
-    bool blockdev = virQEMUCapsGet(priv->qemuCaps, QEMU_CAPS_BLOCKDEV);
     size_t i;
     int rc;
 
@@ -498,13 +495,13 @@ qemuMigrationCookieAddNBD(qemuMigrationCookie *mig,
     mig->nbd->disks = g_new0(struct qemuMigrationCookieNBDDisk, vm->def->ndisks);
     mig->nbd->ndisks = 0;
 
-    if (qemuDomainObjEnterMonitorAsync(driver, vm, priv->job.asyncJob) < 0)
+    if (qemuDomainObjEnterMonitorAsync(vm, priv->job.asyncJob) < 0)
         return -1;
-    if (blockdev)
-        rc = qemuMonitorBlockStatsUpdateCapacityBlockdev(priv->mon, stats);
-    else
-        rc = qemuMonitorBlockStatsUpdateCapacity(priv->mon, stats);
+
+    rc = qemuMonitorBlockStatsUpdateCapacityBlockdev(priv->mon, stats);
+
     qemuDomainObjExitMonitor(vm);
+
     if (rc < 0)
         return -1;
 
@@ -512,14 +509,8 @@ qemuMigrationCookieAddNBD(qemuMigrationCookie *mig,
         virDomainDiskDef *disk = vm->def->disks[i];
         qemuBlockStats *entry;
 
-        if (blockdev) {
-            if (!(entry = virHashLookup(stats, disk->src->nodeformat)))
-                continue;
-        } else {
-            if (!disk->info.alias ||
-                !(entry = virHashLookup(stats, disk->info.alias)))
-                continue;
-        }
+        if (!(entry = virHashLookup(stats, disk->src->nodeformat)))
+            continue;
 
         mig->nbd->disks[mig->nbd->ndisks].target = g_strdup(disk->dst);
         mig->nbd->disks[mig->nbd->ndisks].capacity = entry->capacity;
@@ -1466,7 +1457,7 @@ qemuMigrationCookieFormat(qemuMigrationCookie *mig,
     }
 
     if ((flags & QEMU_MIGRATION_COOKIE_NBD) &&
-        qemuMigrationCookieAddNBD(mig, driver, dom) < 0)
+        qemuMigrationCookieAddNBD(mig, dom) < 0)
         return -1;
 
     if (flags & QEMU_MIGRATION_COOKIE_STATS &&
